from rl.agent import Agent
from rl.gadget import *
import random


class CustomAgent(Agent):
    def __init__(self, seed=None, name="Foo-bar"):
        super(CustomAgent, self).__init__(seed, name)
        self.gadgets = [AverageRewardGadget()]

    def start(self, state, action_set):
        super(CustomAgent, self).start(state, action_set)
        return self.policy(state, action_set)

    def policy(self, state, action_set):
        super(CustomAgent, self).policy(state, action_set)
        rare_actions = self.gadgets[0].rare_actions(state, action_set, bound=10)
        if rare_actions:
            self.prev_action = random.choice(rare_actions)
        else:
            best_rewards = self.gadgets[0].highest_average_reward(state, action_set)
            self.prev_action = random.choice(best_rewards)

        return self.prev_action

    def step(self, past_reward, state, action_set):
        super(CustomAgent, self).step(past_reward, state, action_set)
        return self.policy(state, action_set)

    def finish(self):
        super(CustomAgent, self).finish()

    def __str__(self):
        return "Custom" + str(super(CustomAgent, self))


class AssignmentSolutionAgent(Agent):
    def __init__(self, seed=None, name="AssignmentSolution"):
        super(AssignmentSolutionAgent, self).__init__(seed, name)
        self.gadgets = []

    def start(self, state, action_set):
        super(AssignmentSolutionAgent, self).start(state, action_set)
        return self.policy(state, action_set)

    def policy(self, state, action_set):
        super(AssignmentSolutionAgent, self).policy(state, action_set)

        # Assignment, part III: design an agent to falsify a formula in an environment.
        # You only really need to change this policy method. By running `python -m rl.gym` you can check your
        # falsification score through 5 runs of the navigation benchmark. If you like, you can also mess with
        # the code for gym.py to compare your agent against other agents we saw in class.
        self.prev_action = random.choice(action_set)

        return self.prev_action

    def step(self, past_reward, state, action_set):
        super(AssignmentSolutionAgent, self).step(past_reward, state, action_set)
        return self.policy(state, action_set)

    def finish(self):
        super(AssignmentSolutionAgent, self).finish()

    def __str__(self):
        return "My" + str(super(AssignmentSolutionAgent, self))
